"""Defines BUILD_TARGET_MAP"""
from .base import BuildTarget
from .behave_ import PythonBehaveTestPackage
from .binary import PythonBinaryPackage
from .lambda_function import PythonLambdaPackage
from .library import PythonLibraryPackage
from .migration import AlembicMigrationPackage
from .py2sfn_project import PY2SFNProjectPackage
from .python_package import PythonPackage
from .requirement import PythonRequirement
from .test import PythonTestPackage

BUILD_TARGET_MAP = {
    "behave": PythonBehaveTestPackage,
    "binary": PythonBinaryPackage,
    "lambda": PythonLambdaPackage,
    "library": PythonLibraryPackage,
    "migration": AlembicMigrationPackage,
    "py2sfn_project": PY2SFNProjectPackage,
    "requirement": PythonRequirement,
    "test": PythonTestPackage,
}



"""Contains PythonLibraryPackage class"""
import ast

from .python_package import PythonPackage


class PythonLibraryPackage(PythonPackage):
    """Represents a Python library package build target in Pants

    A library is a package that is meant to be imported by other packages. It is not
    meant to be run as a script.
    """

    def generate_build_file_ast_node(self) -> ast.Module:
        """Generate a Pants BUILD file as an AST module node"""
        body = []
        # check if we have a resource target
        if self.config.resource_glob_path:
            body.append(self._generate_python_library_resources_ast_node())
        # Add in the lib
        body.append(
            self._generate_python_library_ast_node(
                globs_path=f"{self.package_name}/**/*.py"
            )
        )
        node = ast.Module(body=body)
        return node



"""Utility functions"""

import ast
from pathlib import Path
import subprocess
from typing import Set

import astor
import black


class ModuleImportVisitor(ast.NodeVisitor):
    """AST visitor for collecting imports from a module"""

    def __init__(self) -> None:
        self.imports = set()

    def visit_ImportFrom(self, node) -> None:
        """Collect an import when specified as `from foo import bar`"""
        if node.module is not None:
            self.imports.add(node.module.split(".")[0])

    def visit_Import(self, node) -> None:
        """Collect an import when specified as `import foo`"""
        self.imports.add(node.names[0].name.split(".")[0])


def gather_dependencies_from_module(path: str) -> Set[str]:
    """Gather a set of other build targets from a single Python module

    Args:
        path: Python module path

    Returns:
        set of package names that were imported in the module

    """
    with open(path) as f:
        contents = f.read()
    tree = ast.parse(contents)
    visitor = ModuleImportVisitor()
    visitor.visit(tree)
    return visitor.imports


def write_build_file(tree: ast.Module, build_file_path: str) -> None:
    """Format and write a BUILD file from an AST module tree.

    The final output is auto-formatted using Black.

    Args:
        tree: AST module for the BUILD file
        build_file_path: Output path to write the BUILD file

    """
    build_text = black.format_str(
        astor.to_source(tree), mode=black.FileMode(line_length=88)
    )
    with open(build_file_path, "w") as f:
        f.write(build_text)


def get_git_top_level_path() -> Path:
    """Get the path of the git repository where this CLI is running"""
    proc = subprocess.run(
        ["git", "rev-parse", "--show-toplevel"], check=True, stdout=subprocess.PIPE
    )
    output = proc.stdout.decode("utf-8").strip()
    return Path(output)



"""Contains PythonTestPackage class"""
import ast

from .python_package import PythonPackage
from .requirement import PythonRequirement


class PythonTestPackage(PythonPackage):
    """Represents a Python test package build target in Pants"""

    def __init__(self, *args, code_target_package_name: str = None, **kwargs) -> None:
        """
        Args:
            code_target_package_name: Package name for the code package that this test
                package is testing
        """
        super().__init__(*args, **kwargs)
        self.code_target_package_name = code_target_package_name

    def _generate_python_tests_ast_node(self) -> ast.Expr:
        """Generate an AST node for a python_tests Pants target"""
        keywords = [
            ast.keyword(
                arg="dependencies",
                value=ast.List(elts=[ast.Str(f":{self.rendered_package_name}")]),
            ),
            ast.keyword(arg="sources", value=ast.List(elts=[ast.Str("**/*.py")])),
            self._tags_keyword,
        ]
        if self.config.include_test_coverage:
            if self.code_target_package_name is None:
                # Go through the python_library dependencies and get the package names
                # Do not include 3rdparty packages
                code_target_package_names = {
                    d.package_name
                    for d in self.dependencies
                    if not isinstance(d, PythonRequirement)
                    and d.config.include_test_coverage
                }
                # Build the coverage based on the dependency packages
                elements = [ast.Str(pkg) for pkg in sorted(code_target_package_names)]
            else:
                elements = [ast.Str(self.code_target_package_name)]
            keywords.append(ast.keyword(arg="coverage", value=ast.List(elts=elements)))
        node = ast.Expr(
            value=ast.Call(func=ast.Name(id="python_tests"), args=[], keywords=keywords)
        )
        return node

    def generate_build_file_ast_node(self) -> ast.Module:
        """Generate a Pants BUILD file as an AST module node"""
        body = [
            self._generate_python_library_ast_node(
                name=self.rendered_package_name, include_extra_dependencies=True
            ),
            self._generate_python_tests_ast_node(),
            self._generate_python_binary_wrapper_node(
                "unittest", dependencies=[f":{self.rendered_package_name}"]
            ),
        ]
        if self.config.generate_pytest_binary:
            body.append(
                self._generate_python_binary_wrapper_node(
                    "pytest", dependencies=[f":{self.rendered_package_name}"]
                )
            )
        if self.config.resource_glob_path:
            body.append(self._generate_python_library_resources_ast_node())
        node = ast.Module(body=body)
        return node



"""Contains PythonLambdaPackage class"""
import ast
import logging

from ..exceptions import NoConsoleScriptFound  # noqa
from .binary import PythonBinaryPackage  # noqa

logger = logging.getLogger(__name__)


class PythonLambdaPackage(PythonBinaryPackage):
    """Represents a Python Lambda Function build target in Pants

    The Lambda build target is similar to a binary but has a python_awslambda target.
    """

    @property
    def dependency_target(self) -> str:
        """Returns the representation of this target in another target's dependencies

        It's not good practice to depend on a server/binary package (apart from
        tests), but if that happens we need to set the dependency target to
        `:lib` so it correctly references the `python_library` target.
        """
        return f"{self.key}:lib"

    def _generate_python_binary_cli_ast_node(self) -> ast.Expr:
        """Generate an AST node for a python_binary Pants target that runs lambda_handler.py"""  # noqa
        node = ast.Expr(
            value=ast.Call(
                func=ast.Name(id="python_binary"),
                args=[],
                keywords=[
                    ast.keyword(arg="name", value=ast.Str("bin")),
                    ast.keyword(
                        arg="dependencies", value=ast.List(elts=[ast.Str(":lib")])
                    ),
                    ast.keyword(
                        arg="sources",
                        value=ast.List(
                            elts=[ast.Str(f"{self.package_name}/lambda_handler.py")]
                        ),
                    ),
                    ast.keyword(
                        arg="zip_safe", value=ast.NameConstant(self.config.zip_safe)
                    ),
                    self._tags_keyword,
                ],
            )
        )
        return node

    def _generate_python_lambda_ast_node(self) -> ast.Expr:
        """Generate an AST node for a python_awslambda Pants target"""
        binary_name, _ = self._parse_entry_point()
        node = ast.Expr(
            value=ast.Call(
                func=ast.Name(id="python_awslambda"),
                args=[],
                keywords=[
                    ast.keyword(arg="name", value=ast.Str(binary_name)),
                    ast.keyword(
                        arg="dependencies", value=ast.List(elts=[ast.Str(":bin")])
                    ),
                    ast.keyword(
                        arg="handler",
                        value=ast.Str(
                            f"{self.package_name}.lambda_handler:lambda_handler"
                        ),
                    ),
                    ast.keyword(
                        arg="runtime", value=ast.Str(self.config.default_python_runtime)
                    ),
                    self._tags_keyword,
                ],
            )
        )
        return node

    def generate_build_file_ast_node(self) -> ast.Module:
        """Generate a Pants BUILD file as an AST module node"""
        body = [
            self._generate_python_library_ast_node(
                name="lib", globs_path=f"{self.package_name}/**/*.py"
            ),
            self._generate_python_binary_cli_ast_node(),
            self._generate_python_lambda_ast_node(),
        ]
        if self.config.generate_local_binary:
            body.append(self._generate_python_binary_local_ast_node())
        node = ast.Module(body=body)
        return node



"""Contains PY2SFNProjectPackage class"""
import ast
import logging
from pathlib import Path
from typing import Dict, List, Tuple

import astor
from fluxio_parser import parse_project_tree

from .base import BuildTarget
from .python_package import PythonPackage

logger = logging.getLogger(__name__)


class PY2SFNProjectPackage(PythonPackage):
    """Represents a py2sfn project build target in Pants"""

    def find_target_paths(self) -> List[Tuple["PythonPackage", str]]:
        """Find a list of target paths for parsing dependencies

        This implementation returns only the project.sfn file since that defines all the
        dependencies.

        Returns:
            list of tuples with items:
            * the current target (to indicate the owner)
            * the path to the file that will be parsed

        """
        return [(self, str(Path(self.build_dir).joinpath("project.sfn")))]

    def set_extra_dependencies(self, targets: Dict[str, BuildTarget]) -> None:
        """Set extra dependencies on the target

        We need to parse the project.sfn file to add dependencies on the packages
        specified in the ECS worker `spec` attribute.

        Args:
            targets: map of package name to instance of BuildTarget

        """
        project_sfn_file = Path(self.build_dir).joinpath("project.sfn")
        logger.debug(f"Parsing {project_sfn_file}")
        tree = astor.code_to_ast.parse_file(project_sfn_file)
        script_visitor = parse_project_tree(tree)
        for task_visitor in script_visitor.task_visitors.values():
            if task_visitor.attributes["service"] == "ecs:worker":
                # The `spec` attribute is formatted like `my_package.module:ClassName`.
                # We want to extract "my_package"
                module, _ = task_visitor.attributes["spec"].split(":")
                package_name = module.split(".")[0]
                logger.debug(f"Found ecs:worker dependency on {package_name}")
                self.add_dependency(targets, package_name)

    def _generate_generic_target_node(self) -> ast.Expr:
        """Generate an AST node for a generic ``target`` Pants target"""
        node = ast.Expr(
            value=ast.Call(
                func=ast.Name(id="target"),
                args=[],
                keywords=[self._get_dependencies_keyword(), self._tags_keyword],
            )
        )
        return node

    def generate_build_file_ast_node(self) -> ast.Module:
        """Generate a Pants BUILD file as an AST module node"""
        node = ast.Module(body=[self._generate_generic_target_node()])
        return node



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypants', 'pypants.build_targets', 'pypants.generators']

package_data = \
{'': ['*']}

install_requires = \
['Click>=7.0',
 'astor>=0.7.1,<1',
 'black==19.3b0',
 'cookiecutter==1.7.0',
 'fluxio-parser>=0.1.1,<0.2.0',
 'networkx>=2.2,<3',
 'python-slugify>=1.2.0,<2']

entry_points = \
{'console_scripts': ['pypants = pypants.cli:cli']}

setup_kwargs = {
    'name': 'pypants',
    'version': '1.30.2.dev1',
    'description': 'CLI for working with Python packages and BUILD files in a Pants monorepo',
    'long_description': '# pypants\n\n[![](https://img.shields.io/pypi/v/pypants.svg)](https://pypi.org/pypi/pypants/) [![License](https://img.shields.io/badge/License-BSD%203--Clause-blue.svg)](https://opensource.org/licenses/BSD-3-Clause)\n\nCLI for working with Python packages and BUILD files in a [Pants](https://www.pantsbuild.org/) monorepo.\n\nFeatures:\n\n- Auto-generate BUILD files based on the package type and import statements\n- Generate new Python package folders through an interactive CLI\n- Compute a topologically-sorted list of dependencies for a given Python build target\n\nTable of Contents:\n\n- [Installation](#installation)\n- [Guide](#guide)\n  - [Commands](#commands)\n  - [Project Configuration](#project-configuration)\n  - [Package Configuration](#package-configuration)\n  - [Package Types](#package-types)\n  - [Registering Extra Targets](#registering-extra-targets)\n  - [Package Generators](#package-generators)\n- [Development](#development)\n\n## Installation\n\npypants requires Python 3.6 or above\n\n```bash\npip install pypants\n```\n\n## Guide\n\n### Commands\n\n#### `pypants process-requirements`\n\nUpdate `3rdparty/python/import-map.json` using the entries in `3rdparty/python/requirements.txt`. All this does is convert the published package name to an import name. Execute this command when you add a new requirement to `requirements.txt`.\n\n#### `pypants process-packages`\n\nAuto-generate all relevant BUILD files in the project/repo. You should execute this command in a git pre-commit or pre-push hook so your BUILD files are kept up to date. You can also run it on demand after you add a new dependency to an internal package.\n\n#### `pypants generate-package`\n\nStarts an interactive CLI that generates a new package folder. This depends on the package generators you registered.\n\n### Project Configuration\n\nTo configure your project, add a file named `.pypants.cfg` to the root of your Git repo and paste the example below. You should define the `top_dirs` option at a minimum.\n\n```ini\n[project]\n\n; **************\n; COMMON OPTIONS\n; **************\n\n; REQUIRED: Top-level directories to search for Python packages. These are relative\n; to your project/repo root. This is a JSON list of strings.\ntop_dirs = ["."]\n\n; Prefix to use for names of packages generated by pypants. e.g. foobar_\n; python_package_name_prefix =\n\n; Never look for or process files in these directories. This is a JSON list of\n; strings. e.g. ["node_modules", "generators"]\n; ignore_dirs = []\n\n; ****************\n; UNCOMMON OPTIONS\n; ****************\n\n; Set of target package names to ignore when collecting build targets. This is a\n; JSON list of strings.\n; ignore_targets = []\n\n; Path to the location of the import-map.json file relative to the project root\n; third_party_import_map_path = 3rdparty/python/import-map.json\n\n; Path to the requirements.txt relative to the project root. The default value is\n; the default that Pants uses.\n; third_party_requirements_path = 3rdparty/python/requirements.txt\n```\n\nBesides the JSON lists, other options are parsed with Python\'s built-in [ConfigParser](https://docs.python.org/3/library/configparser.html).\n\n### Package Configuration\n\n`pypants` currently expects the Python package to be structured like:\n\n```txt\nmypackage/\n├── setup.py\n├── .pypants.cfg <---- this is the pypants config file\n├── src/\n    ├── BUILD <---- pypants will generate this file\n    ├── mypackage/\n        ├── __init__.py\n        ├── ...source code...\n├── tests/\n    ├── unit/\n        ├── BUILD <---- pypants will generate this file\n        ├── ...unit tests...\n    ├── functional/\n        ├── BUILD <---- pypants will generate this file\n        ├── ...functional tests...\n```\n\nTo configure each package, add a file named `.pypants.cfg` to the package folder and paste the example below. You should define the `type` option at a minimum.\n\n```ini\n[package]\n\n; **************\n; COMMON OPTIONS\n; **************\n\n; REQUIRED: Package type. See Package Types section for available values.\ntype = library\n\n; ****************\n; UNCOMMON OPTIONS\n; ****************\n\n; Extra set of dependencies to include in the python_library target. This is a\n; JSON list of strings.\n; extra_dependencies = []\n\n; Extra set of tags to include in the Pants build targets. This is a JSON list of\n; strings.\n; extra_tags = []\n\n; Flag denoting whether to generate a BUILD file.\n; generate_build_file = true\n\n; Flag denoting whether to generate a python_binary target for local.py. This is\n; essentially an extra entry point. It\'s only used for specific package types.\n; generate_local_binary = false\n\n; Flag denoting whether to include a python_binary target for pytest\n; generate_pytest_binary = false\n\n; Flag denoting whether to include a coverage attribute on pytest targets\n; include_test_coverage = true\n```\n\n### Package Types\n\nEach of the package types will result in a different BUILD file.\n\n#### `library`\n\nThe BUILD file for internal Python libraries has one target defined. For example:\n\n```python\npython_library(\n    dependencies=[\n        "3rdparty/python:arrow",\n        "3rdparty/python:isoweek",\n        "lib/code/src",\n        "lib/logger/src",\n    ],\n    sources=["my_library/**/*"],\n    tags={"code", "lib", "python"},\n)\n```\n\nThere is no name provided so this target can be referenced just by its containing folder path. In this case it would be `"<TOPDIR>/my_library/src"`.\n\n#### `binary`\n\nA binary target can be used for executable scripts (CLIs and servers) and usually depend on internal libraries. The BUILD has a library and binary target defined:\n\n```python\npython_library(\n    name="lib",\n    dependencies=[\n        "3rdparty/python:boto3",\n        "3rdparty/python:cfn-flip",\n        "3rdparty/python:Click",\n        "3rdparty/python:jsonschema",\n        "lib/logger/src",\n    ],\n    sources=["cli_deploy/**/*"],\n    tags={"apps", "code", "python"},\n)\npython_binary(\n    name="deploy",\n    dependencies=[":lib"],\n    source="cli_deploy/cli.py",\n    tags={"apps", "code", "python"},\n)\n```\n\n- The `python_library` target is pretty much the same as an internal Python library package\n- The `python_binary` target defines an explicit name. This is because when we go to build the PEX file, we want to define the filename. In this example, running `./pants binary apps/cli_deploy/src:deploy` will result in `dist/deploy.pex`.\n- The only dependency for the binary should be the library. The library will then include all the dependencies.\n- `source` points to the entry point of the binary. This module should handle the `if __name__ == "__main__"` condition to kick off the script.\n\n#### `test`\n\npypants looks for subfolders named unit, functional, or component within a package\'s `tests/` folder. The BUILD file for test folders have a few targets defined. For example:\n\n```python\npython_library(\n    name="lib/time_utils/tests/unit",\n    dependencies=[\n        "3rdparty/python:arrow",\n        "lib/python_core/src",\n        "lib/time_utils/src"\n    ],\n    sources=["**/*"],\n    tags={"lib", "python", "tests", "unit"},\n)\npython_tests(\n    dependencies=[":lib/time_utils/tests/unit"],\n    sources=["**/*.py"],\n    tags={"lib", "python", "tests", "unit"},\n)\npython_binary(\n    name="unittest",\n    entry_point="unittest",\n    dependencies=[":lib/time_utils/tests/unit"]\n)\n```\n\n- The `python_library` target is mostly here to define the unit tests dependencies in a single place so the other two targets can point to it\n- The `python_tests` target lets us run pytest against the test files that match `**/*.py`\n- The `python_binary` target lets us run the unittest module directly. We won\'t actually package up this target via `./pants binary`. Setting the entry_point to `"unittest"` is essentially the same as running `python -m unittest test_something.py` from the command line.\n\n#### `lambda_function`\n\nThe BUILD file for the Lambda handler contains a special-purpose build target: `python_awslambda`. This target is a wrapper around [lambdex](https://github.com/wickman/lambdex). It creates a PEX like the `python_binary` target (you can execute it) but it modifies the PEX to work with a Lambda Function. For example:\n\n```python\npython_library(\n    name="my-lambda-lib",\n    sources=["lambda_handler/**/*"],\n    dependencies=[\n        "3rdparty/python:requests",\n        "lib/logger/src",\n    ],\n)\npython_binary(\n    name="my-lambda-bin",\n    source="lambda_handler/lambda_handler.py",\n    dependencies=[":my-lambda-lib"],\n)\npython_awslambda(\n    name="my-lambda",\n    binary=":my-lambda-bin",\n    handler="lambda_handler.lambda_handler:lambda_handler",\n)\n```\n\nThis BUILD file will be placed in the same folder as the `.pypants.cfg` file.\n\n#### `migration`\n\nThe BUILD file for an [Alembic](https://alembic.sqlalchemy.org/) migration uses the `python_app` target to include the loose version files:\n\n```python\npython_library(\n    name="lib",\n    dependencies=[\n        "3rdparty/python:alembic",\n        "3rdparty/python:SQLAlchemy",\n        "lib/core/src",\n    ],\n    sources=["**/*"],\n    tags={"code", "db", "migration", "python"},\n)\npython_binary(name="alembic", entry_point="alembic.config", dependencies=[":lib"])\npython_app(\n    name="migrations-my-database-name",\n    archive="tar",\n    binary=":alembic",\n    bundles=[\n        bundle(fileset=["alembic.ini"]),\n        bundle(fileset=["env.py"]),\n        bundle(fileset=["versions/*.py"]),\n    ],\n    tags={"code", "db", "migration", "python"},\n)\n```\n\nThis BUILD file will be placed in the same folder as the `.pypants.cfg` file.\n\n#### `behave`\n\nThe BUILD file for a [behave](https://behave.readthedocs.io/en/latest/) test package includes a library target with test dependencies and a binary target that wraps behave. For example:\n\n```python\npython_library(\n    name="lib",\n    dependencies=[\n        "3rdparty/python:requests",\n        "lib/application_config/src",\n    ],\n    sources=["**/*"],\n    tags={"integration", "python", "tests", "tests-integration"},\n)\npython_binary(\n    source="behave_cli.py",\n    dependencies=[":lib"],\n    tags={"integration", "python", "tests", "tests-integration"},\n)\n```\n\nThis BUILD file will be placed in the same folder as the `.pypants.cfg` file.\n\nThe `behave_cli.py` source references a wrapper script that you should add to the folder:\n\n```python\n"""Programmatic entrypoint to running behave from the command line"""\nimport os\nimport sys\n\nfrom behave.__main__ import main as behave_main\n\nif __name__ == "__main__":\n    cwd = os.getcwd()\n    os.chdir(os.path.dirname(__file__))\n    try:\n        exit_code = behave_main(sys.argv[1:])\n    finally:\n        os.chdir(cwd)\n        sys.exit(exit_code)\n```\n\n#### `py2sfn_project`\n\npy2sfn is a framework that simplifies the creation and deployment of workflows to [AWS Step Functions](https://docs.aws.amazon.com/step-functions/latest/dg/welcome.html). The BUILD file for a project only includes a generic target with the set of task dependencies:\n\n```python\ntarget(\n    dependencies=[\n        "stepfunctions/projects/example-project/tasks/lambda_fetchjoke/src:lib",\n        "stepfunctions/projects/example-project/tasks/lambda_generatelist/src:lib",\n        "stepfunctions/projects/example-project/tasks/lambda_rankcharactersbyjoke/src:lib",\n    ],\n    tags={"py2sfn-project", "python", "stepfunctions/projects"},\n)\n```\n\nThis BUILD file will be placed in the same folder as the `.pypants.cfg` file.\n\n### Registering Extra Targets\n\nIf your project contains internal packages that don\'t aren\'t represented cleanly by the `.pypants.cfg` file, you can register extra targets programmatically.\n\n1. In your repo, create a new file at `.pypants/targets.py`\n1. Define a top-level function called `register_extra_targets`. Within that function, instantiate your extra build targets and return a dictionary that maps package name to `BuildTarget`.\n\nFor example, if you have several Alembic database folders:\n\n```python\n"""Module that defines extra pypants build targets"""\nfrom typing import Dict\n\nfrom pypants.config import PROJECT_CONFIG\nfrom pypants.build_targets import AlembicMigrationPackage\n\n\ndef register_extra_targets() -> Dict[str, "pypants.build_targets.base.PythonPackage"]:\n    """Register extra targets specific to MyProject"""\n    targets = {}\n\n    # Register task targets for Alembic database migration targets\n    #\n    # * For migrations, this searches db/ looking for eny.py files. If it finds one,\n    #   it means we\'ve found an Alembic migration folder and can register a build\n    #   target.\n    env_py_paths = PROJECT_CONFIG.config_dir_path.joinpath("db").glob("**/env.py")\n    for env_py_path in env_py_paths:\n        alias = env_py_path.parent.name.replace("_db", "").replace("_", "-")\n        package_name = f"migrations-{alias}"\n        target = AlembicMigrationPackage(\n            target_type="code",\n            build_template="migration",\n            top_dir_name="db",\n            package_dir_name=env_py_path.parent.name,\n            package_path=str(env_py_path.parent),\n            package_name=package_name,\n            build_dir=str(env_py_path.parent),\n            extra_tags={"migration"},\n        )\n        targets[package_name] = target\n\n    return targets\n```\n\n### Package Generators\n\nThe `generate-package` command can be used to create a new package on disk. It sources package "generators" (folders that define the package boilerplate) from the `.pypants/generators` folder in your repo. To create a new package generator, copy one of the folders from [`examples/generators/`](examples/generators/) to `<your repo>/.pypants/generators/<name>` and modify it as needed. The generators use a tool called [cookiecutter](https://github.com/cookiecutter/cookiecutter) to rendere templates.\n\n## Development\n\nIf you\'re working on pypants locally and want to test out how your changes impact your target project:\n\n1. Install poetry: `pip install poetry`\n2. Install dependencies: `poetry install`\n3. Activate the virtualenv: `poetry shell`\n4. Run `pytest` then `poetry build`\n',
    'author': 'Jonathan Drake',
    'author_email': 'jdrake@narrativescience.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/NarrativeScience/pypants',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



"""Command line interface"""
import logging
import re

import click
from slugify import slugify

from .config import PROJECT_CONFIG
from .generators.load import create_package_generator, PACKAGE_GENERATOR_MAP
from .process_packages import PackageProcessor
from .requirements import update_third_party_import_map

logger = logging.getLogger(__name__)


@click.group()
@click.option(
    "--log-level",
    type=click.Choice(["NOTSET", "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]),
    default="INFO",
)
def cli(log_level: str):
    """Base CLI command"""
    logging.basicConfig(level=getattr(logging, log_level.upper()))


@cli.command()
def process_packages() -> None:
    """Generate BUILD files"""
    processor = PackageProcessor()
    processor.register_packages()
    processor.ensure_no_circular_imports()
    processor.generate_build_files()
    logger.info("Done")


@cli.command()
@click.option("--target", help="Pants build target")
@click.option(
    "--include-3rdparty/--no-include-3rdparty",
    default=False,
    help="Include or exclude 3rdparty requirements in the list of dependencies",
)
def compute_package_dependencies(target: str, include_3rdparty: bool) -> None:
    """Compute a topologically-sorted list of dependencies for a given target"""
    processor = PackageProcessor()
    processor.register_packages()
    dependencies = processor.compute_package_dependencies(
        target, include_3rdparty=include_3rdparty
    )
    # Print each of the dependency's build directory paths to stdout so we can parse
    # the list with bash in a CI job
    for d in dependencies:
        print(d.build_dir)


@cli.command()
def process_requirements() -> None:
    """Update 3rdparty/python/import-map.json from requirements.txt"""
    update_third_party_import_map()


def _validate_package_nickname(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    """Validate the package nickname and return a coerced form.

    This is meant to be passed to the ``callback`` argument of a ``@click.option()``
    decorator.

    Args:
        ctx: Click context object
        param: Click parameter object
        value: Value of the parameter, i.e. the nickname

    Returns:
        Coerced nickname

    """
    return slugify(
        re.sub(
            rf"^(?:{PROJECT_CONFIG.python_package_name_prefix}|cli_|python_)+(.*)$",
            r"\1",
            value,
        )
    )


@cli.command()
@click.option(
    "--type",
    "package_type",
    help="Package type",
    type=click.Choice(list(PACKAGE_GENERATOR_MAP.keys())),
    prompt=True,
)
@click.option(
    "--nickname",
    "nickname",
    callback=_validate_package_nickname,
    help="Package nickname",
    prompt="Enter a package nickname. This will be used to folder names and other"
    " identifiers. Examples: deploy | config_server ",
)
@click.option(
    "--title",
    help="Title of the package. Examples: Deploy CLI | Configuration Server",
    prompt="Enter a title for the package. We'll include this in the README."
    " Examples: Deploy CLI | Configuration Server ",
)
@click.option(
    "--description",
    help="Short description of the package",
    prompt="Enter a short description for the package. We'll include this in the"
    " README ",
)
def generate_package(
    package_type: str, nickname: str, title: str, description: str
) -> None:
    """Generate a new package folder"""
    generator = create_package_generator(package_type, nickname, title, description)
    generator.generate()
    generator.create_build_file()
    click.echo()
    click.secho("Your new package was successfully generated:", fg="green", bold=True)
    click.echo(generator.package_dir)
    generator.print_extra_help()


if __name__ == "__main__":
    cli()
